@@ -25,6 +25,13 @@ import stat
 
 using_subversion = True
 
+#
+# Working around error with PYTHONHOME
+#
+if 'PYTHONHOME' in os.environ:
+    del os.environ['PYTHONHOME']
+    print "WARNING: ignoring the value of the PYTHONHOME environment variable!  This value can corrupt the virtual python installation."
+
 #
 # The following taken from PyUtilib
 #
@@ -98,6 +105,17 @@ def parse_version(s):
 #   9.3.2
 #   8.28.rc1
 #
+def guess_release(svndir):
+    if using_subversion:
+        output = commands.getoutput('svn ls '+svndir)
+        if output=="":
+            return None
+        #print output
+        versions = []
+# This allows for parsing version with the syntax:
+#   9.3.2
+#   8.28.rc1
+#
 def guess_release(svndir):
     if using_subversion:
         output = commands.getoutput('svn ls '+svndir)
@@ -402,14 +420,35 @@ class Repository(object):
         if os.path.exists(dir):
             print "     No checkout required"
             print "-----------------------------------------------------------------"
+        elif not using_subversion:
+                print ""
+                print "Error: Cannot checkout software %s with subversion." % self.name
+                print "A problem was detected executing subversion commands."
+                sys.exit(1)
         else:
             print "-----------------------------------------------------------------"
-            self.run([self.svn]+self.svn_username+[Repository.svn_get,'-q',self.pkgdir+self.rev, dir])
+            try:
+                self.run([self.svn]+self.svn_username+[Repository.svn_get,'-q',self.pkgdir+self.rev, dir])
+            except OSError, err:
+                print ""
+                print "Error checkout software %s with subversion at %s" % (self.name,self.pkgdir+self.rev)
+                print str(err)
+                sys.exit(1)
         if install:
-            if self.dev:
-                self.run([self.python, 'setup.py', 'develop'], dir=dir)
-            else:
-                self.run([self.python, 'setup.py', 'install'], dir=dir)
+            try:
+                if self.dev:
+                    if offline:
+                        self.run([self.python, 'setup.py', 'develop', '--no-deps'], dir=dir)
+                    else:
+                        self.run([self.python, 'setup.py', 'develop'], dir=dir)
+                else:
+                    self.run([self.python, 'setup.py', 'install'], dir=dir)
+            except OSError, err:
+                print ""
+                print "Error installing software %s from source using the setup.py file." % self.name
+                print "This is probably due to a syntax or configuration error in this package."
+                print str(err)
+                sys.exit(1)
 
     def update_trunk(self, dir=None):
         self.find_pkgroot(trunk=True)
@@ -439,68 +478,9 @@ class Repository(object):
         else:
             self.run([self.python, 'setup.py', 'install'], dir=dir)
 
-    def Xsdist_trunk(self, format='zip'):
-        if self.trunk is None:
-            if not self.pypi is None:
-                self.easy_install()
-            elif not self.stable is None:
-                self.sdist_stable(format=format)
-            else:
-                self.sdist_release(format=format)
-        else:
-            self.pkgdir=self.trunk
-            self.pkgroot=self.trunk_root
-            print "-----------------------------------------------------------------"
-            print "  Checking out source for package",self.name
-            print "     Subversion dir: "+self.trunk
-            print "-----------------------------------------------------------------"
-            self.run([self.svn]+self.svn_username+[Repository.svn_get,'-q',self.trunk, 'pkg'+self.name+self.rev])
-            self.run([self.python, 'setup.py', 'sdist','-q','--dist-dir=..', '--formats='+format], dir='pkg'+self.name)
-            os.chdir('..')
-            rmtree('pkg'+self.name)
-
-    def Xsdist_stable(self, format='zip'):
-        if self.stable is None:
-            if not self.pypi is None:
-                self.easy_install()
-            elif not self.release is None:
-                self.install_release()
-            elif not self.trunk is None:
-                self.install_trunk()
-        else:
-            self.pkgdir=self.stable
-            self.pkgroot=self.stable_root
-            print "-----------------------------------------------------------------"
-            print "  Checking out source for package",self.name
-            print "     Subversion dir: "+self.stable
-            print "     Source dir:     "+dir
-            print "-----------------------------------------------------------------"
-            self.run([self.svn]+self.svn_username+[Repository.svn_get,'-q',self.stable, dir])
-            self.run([self.python, 'setup.py', 'develop'], dir=dir)
-
-    def Xsdist_release(self, dir=None):
-        if self.release is None:
-            if not self.pypi is None:
-                self.easy_install()
-            elif not self.stable is None:
-                self.install_stable()
-            elif not self.trunk is None:
-                self.install_trunk()
-        else:
-            self.pkgdir=self.release
-            self.pkgroot=self.release_root
-            print "-----------------------------------------------------------------"
-            print "  Checking out source for package",self.name
-            print "     Subversion dir: "+self.release
-            print "     Source dir:     "+dir
-            print "-----------------------------------------------------------------"
-            self.run([self.svn]+self.svn_username+[Repository.svn_get]+self.rev+['-q',self.release, dir])
-            self.run([self.python, 'setup.py', 'install'], dir=dir)
-
     def easy_install(self, install, preinstall, dir, offline):
         try:
             if install:
-                #self.run([self.easy_install_path, '-q', self.pypi])
                 if offline:
                     self.run([self.python, 'setup.py', 'install'], dir=dir)
                 else:
@@ -509,9 +489,10 @@ class Repository(object):
                 if not os.path.exists(dir):
                     self.run(self.easy_install_path + ['-q', '--editable', '--build-directory', '.', self.pypi], dir=os.path.dirname(dir))
         except OSError, err:
-            print "-----------------------------------------------------------------"
-            print "Warning!!! Ignoring easy_install error '%s'" % str(err)
-            print "-----------------------------------------------------------------"
+            print ""
+            print "Error installing package %s with easy_install" % self.name
+            print str(err)
+            sys.exit(1)
 
     def easy_upgrade(self):
         self.run(self.easy_install_path + ['-q', '--upgrade', self.pypi])
@@ -563,6 +544,7 @@ class Installer(object):
         self.sw_dict = {}
         self.cmd_files = []
         self.auxdir = []
+        self.srcdir = None
         self.config=None
         self.config_file=None
         self.README="""
@@ -610,14 +592,20 @@ class Installer(object):
             dest='debug',
             default=False)
 
+        parser.add_option('--release',
+            help='Install release branches of Python software using subversion.',
+            action='store_true',
+            dest='release',
+            default=False)
+
         parser.add_option('--trunk',
-            help='Install trunk branches of Python software.',
+            help='Install trunk branches of Python software using subversion.',
             action='store_true',
             dest='trunk',
             default=False)
 
         parser.add_option('--stable',
-            help='Install stable branches of Python software.',
+            help='Install stable branches of Python software using subversion.',
             action='store_true',
             dest='stable',
             default=False)
@@ -652,6 +640,12 @@ class Installer(object):
             dest='zip',
             default=[])
 
+        parser.add_option('--source', '--src',
+            help='Use packages defined in the specified source directory',
+            action='store',
+            dest='source',
+            default=None)
+
         parser.add_option('--use-pythonpath',
             help="By default, the PYTHONPATH is ignored when installing.  This option allows the 'easy_install' tool to search this path for related Python packages, which are then installed.",
             action='store_true',
@@ -719,6 +713,20 @@ class Installer(object):
         self.logger = Logger([(Logger.level_for_integer(2-verbosity), sys.stdout)])
         logger = self.logger
         #
+        # Determine if the subversion command is available
+        #
+        global using_subversion
+        try:
+            call_subprocess(['svn'+executable_extension,'help'], show_stdout=False)
+        except OSError, err:
+            print ""
+            print "------------------------------------------------"
+            print "WARNING: problems executing subversion commands."
+            print "Subversion is disabled."
+            print "------------------------------------------------"
+            print ""
+            using_subversion = False
+        #
         if options.update and (options.stable or options.trunk):
             self.logger.fatal("ERROR: cannot specify --stable or --trunk when specifying the --update option.")
             sys.exit(1000)
@@ -744,6 +752,11 @@ class Installer(object):
         if len(args) == 0:
             args.append(self.abshome_dir)
         #
+        # Reset the config file if no options are specified
+        #
+        if not self.config_file is None and not (options.trunk or options.stable or options.release):
+            self.config_file = os.path.dirname(self.config_file)+"/pypi.ini"
+        #
         # Parse config files
         #
         if options.update:
@@ -790,6 +803,12 @@ class Installer(object):
             home_dir = args[0]
         self.home_dir = home_dir
         self.abshome_dir = os.path.abspath(home_dir)
+        if options.source is None:
+            self.srcdir = join(self.abshome_dir,'src')
+        else:
+            self.srcdir = os.path.abspath(options.source)
+            if not os.path.exists(self.srcdir):
+                raise ValueError, "Specified source directory does not exist! %s" % self.srcdir
 
     def guess_path(self):
         return None
@@ -849,10 +868,9 @@ class Installer(object):
         # If we're clearing the current installation, then remove a bunch of
         # directories
         #
-        elif options.clear:
-            path = join(self.abshome_dir,'src')
-            if os.path.exists(path):
-                rmtree(path)
+        elif options.clear and not options.source is None:
+            if os.path.exists(self.srcdir):
+                rmtree(self.srcdir)
         #
         # Open up zip files
         #
@@ -884,11 +902,13 @@ class Installer(object):
             INPUT=open(join(self.abshome_dir,'admin',"virtualenv.cfg"),'r')
             options.trunk = INPUT.readline().strip() != 'False'
             options.stable = INPUT.readline().strip() != 'False'
+            options.release = INPUT.readline().strip() != 'False'
             INPUT.close()
         else:
             OUTPUT=open(join(self.abshome_dir,'admin',"virtualenv.cfg"),'w')
             print >>OUTPUT, options.trunk
             print >>OUTPUT, options.stable
+            print >>OUTPUT, options.release
             OUTPUT.close()
             self.write_config( join(self.abshome_dir,'admin','config.ini') )
         #
@@ -896,8 +916,8 @@ class Installer(object):
         #
         if not os.path.exists(join(self.abshome_dir,'dist')):
             os.mkdir(join(self.abshome_dir,'dist'))
-        if not os.path.exists(self.abshome_dir+os.sep+"src"):
-            os.mkdir(self.abshome_dir+os.sep+"src")
+        if not os.path.exists(self.srcdir):
+            os.mkdir(self.srcdir)
         if not os.path.exists(self.abshome_dir+os.sep+"bin"):
             os.mkdir(self.abshome_dir+os.sep+"bin")
         #
@@ -920,10 +940,10 @@ class Installer(object):
         for pkg in self.sw_packages:
             pkg.guess_versions(False)
             if not pkg.install:
-                pkg.find_pkgroot(trunk=options.trunk, stable=options.stable, release=not (options.trunk or options.stable))
+                pkg.find_pkgroot(trunk=options.trunk, stable=options.stable, release=options.release)
                 continue
             if pkg.dev:
-                tmp = join(self.abshome_dir,'src',pkg.name)
+                tmp = join(self.srcdir,pkg.name)
             else:
                 tmp = join(self.abshome_dir,'dist',pkg.name)
             if options.trunk:
@@ -985,10 +1005,10 @@ class Installer(object):
         #
         for pkg in self.sw_packages:
             if not pkg.install:
-                pkg.find_pkgroot(trunk=options.trunk, stable=options.stable, release=not (options.trunk or options.stable))
+                pkg.find_pkgroot(trunk=options.trunk, stable=options.stable, release=options.release)
                 continue
             if pkg.dev:
-                srcdir = join(self.abshome_dir,'src',pkg.name)
+                srcdir = join(self.srcdir,pkg.name)
             else:
                 srcdir = join(self.abshome_dir,'dist',pkg.name)
             if options.trunk:
@@ -1143,7 +1163,6 @@ def get_installer():
     except:
         get_installer.installer = configure( create_installer() )
         return get_installer.installer
-    
 
 #
 # Override the default definition of rmtree, to better handle MSWindows errors
@@ -1200,7 +1219,7 @@ def install_setuptools(py_executable, unzip=False):
             print ""
             print "WARNING: you may need to set your HTTP_PROXY environment variable!"
         print "-----------------------------------------------------------------"
-        sys.exit(0)
+        sys.exit(1)
 
 install_setuptools.use_default=True
 
@@ -1222,11 +1241,27 @@ def install_pip(*args, **kwds):
             print ""
             print "WARNING: you may need to set your HTTP_PROXY environment variable!"
         print "-----------------------------------------------------------------"
-        sys.exit(0)
+        sys.exit(1)
 
 install_pip.use_default=True
 
 
+#
+# This is a monkey patch, to catch errors when a directory cannot be created
+# by virtualenv.
+#
+def mkdir(path):
+    if not os.path.exists(path):
+        logger.info('Creating %s', path)
+        try:
+            os.makedirs(path)
+        except Exception, e:
+            print "Cannot create directory '%s'!" % path
+            print "Verify that you have write permissions to this directory."
+            sys.exit(1)
+    else:
+        logger.info('Directory %s already exists', path)
+
 #
 # The following methods will be called by virtualenv
 #
